@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2012-2016 Open Source Robotics Foundation
+* Copyright (C) 2012 Open Source Robotics Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -29,9 +29,9 @@ using namespace common;
 /////////////////////////////////////////////////
 AudioDecoder::AudioDecoder()
 {
-  this->formatCtx = NULL;
-  this->codecCtx = NULL;
-  this->codec = NULL;
+  this->formatCtx = nullptr;
+  this->codecCtx = nullptr;
+  this->codec = nullptr;
   this->audioStream = 0;
 }
 
@@ -59,20 +59,24 @@ void AudioDecoder::Cleanup()
 #ifdef HAVE_FFMPEG
 bool AudioDecoder::Decode(uint8_t **_outBuffer, unsigned int *_outBufferSize)
 {
-  AVPacket packet, packet1;
+#if LIBAVFORMAT_VERSION_MAJOR < 59
+  AVPacket *packet, packet1;
   int bytesDecoded = 0;
+#else
+  AVPacket *packet;
+#endif
   unsigned int maxBufferSize = 0;
-  AVFrame *decodedFrame = NULL;
+  AVFrame *decodedFrame = nullptr;
 
-  if (this->codec == NULL)
+  if (this->codec == nullptr)
   {
     gzerr << "Set an audio file before decoding.\n";
     return false;
   }
 
-  if (_outBufferSize == NULL)
+  if (_outBufferSize == nullptr)
   {
-    gzerr << "outBufferSize is NULL!!\n";
+    gzerr << "outBufferSize is null!!\n";
     return false;
   }
 
@@ -81,7 +85,7 @@ bool AudioDecoder::Decode(uint8_t **_outBuffer, unsigned int *_outBufferSize)
   if (*_outBuffer)
   {
     delete [] *_outBuffer;
-    *_outBuffer = NULL;
+    *_outBuffer = nullptr;
   }
 
   bool result = true;
@@ -97,14 +101,66 @@ bool AudioDecoder::Decode(uint8_t **_outBuffer, unsigned int *_outBufferSize)
   else
     common::AVFrameUnref(decodedFrame);
 
-  av_init_packet(&packet);
-  while (av_read_frame(this->formatCtx, &packet) == 0)
+  packet = av_packet_alloc();
+  if (!packet)
   {
-    if (packet.stream_index == this->audioStream)
+    gzerr << "Failed to allocate AVPacket" << std::endl;
+    return false;
+  }
+  while (av_read_frame(this->formatCtx, packet) == 0)
+  {
+    if (packet->stream_index == this->audioStream)
     {
+#if LIBAVFORMAT_VERSION_MAJOR >= 59
+      // Inspired from
+      // https://github.com/FFmpeg/FFmpeg/blob/n5.0/doc/examples/decode_audio.c#L71
+
+      // send the packet with the compressed data to the decoder
+      int ret = avcodec_send_packet(this->codecCtx, packet);
+      if (ret < 0)
+      {
+        gzerr << "Error submitting the packet to the decoder" << std::endl;
+        return false;
+      }
+
+      // read all the output frames
+      // (in general there may be any number of them)
+      while (ret >= 0)
+      {
+        ret = avcodec_receive_frame(this->codecCtx, decodedFrame);
+        if (ret == AVERROR(EAGAIN) || ret == AVERROR_EOF)
+        {
+          break;
+        }
+        else if (ret < 0)
+        {
+            gzerr << "Error during decoding" << std::endl;
+            return false;
+        }
+
+        // Total size of the data. Some padding can be added to
+        // decodedFrame->data[0], which is why we can't use
+        // decodedFrame->linesize[0].
+        int size = decodedFrame->nb_samples *
+          av_get_bytes_per_sample(this->codecCtx->sample_fmt) *
+          this->codecCtx->channels;
+
+        // Resize the audio buffer as necessary
+        if (*_outBufferSize + size > maxBufferSize)
+        {
+          maxBufferSize += size * 5;
+          *_outBuffer = reinterpret_cast<uint8_t*>(realloc(*_outBuffer,
+                maxBufferSize * sizeof(*_outBuffer[0])));
+        }
+
+        memcpy(*_outBuffer + *_outBufferSize, decodedFrame->data[0],
+            size);
+        *_outBufferSize += size;
+    }
+#else
       int gotFrame = 0;
 
-      packet1 = packet;
+      packet1 = *packet;
       while (packet1.size)
       {
         // Some frames rely on multiple packets, so we have to make sure
@@ -144,11 +200,12 @@ bool AudioDecoder::Decode(uint8_t **_outBuffer, unsigned int *_outBufferSize)
         packet1.data += bytesDecoded;
         packet1.size -= bytesDecoded;
       }
+#endif
     }
-    AVPacketUnref(&packet);
+    av_packet_unref(packet);
   }
 
-  AVPacketUnref(&packet);
+  av_packet_unref(packet);
 
   // Seek to the beginning so that it can be decoded again, if necessary.
   av_seek_frame(this->formatCtx, this->audioStream, 0, 0);
@@ -182,10 +239,11 @@ bool AudioDecoder::SetFile(const std::string &_filename)
   this->formatCtx = avformat_alloc_context();
 
   // Open file
-  if (avformat_open_input(&this->formatCtx, _filename.c_str(), NULL, NULL) < 0)
+  if (avformat_open_input(&this->formatCtx,
+        _filename.c_str(), nullptr, nullptr) < 0)
   {
     gzerr << "Unable to open audio file[" << _filename << "]\n";
-    this->formatCtx = NULL;
+    this->formatCtx = nullptr;
     return false;
   }
 
@@ -193,11 +251,11 @@ bool AudioDecoder::SetFile(const std::string &_filename)
   av_log_set_level(0);
 
   // Retrieve some information
-  if (avformat_find_stream_info(this->formatCtx, NULL) < 0)
+  if (avformat_find_stream_info(this->formatCtx, nullptr) < 0)
   {
     gzerr << "Unable to find stream info.\n";
     avformat_close_input(&this->formatCtx);
-    this->formatCtx = NULL;
+    this->formatCtx = nullptr;
 
     return false;
   }
@@ -213,7 +271,11 @@ bool AudioDecoder::SetFile(const std::string &_filename)
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #endif
+#if LIBAVFORMAT_VERSION_MAJOR >= 59
+    if (this->formatCtx->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_AUDIO)
+#else
     if (this->formatCtx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO)
+#endif
 #ifndef _WIN32
 # pragma GCC diagnostic pop
 #endif
@@ -227,7 +289,7 @@ bool AudioDecoder::SetFile(const std::string &_filename)
   {
     gzerr << "Couldn't find audio stream.\n";
     avformat_close_input(&this->formatCtx);
-    this->formatCtx = NULL;
+    this->formatCtx = nullptr;
 
     return false;
   }
@@ -237,32 +299,85 @@ bool AudioDecoder::SetFile(const std::string &_filename)
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #endif
+#if LIBAVFORMAT_VERSION_MAJOR < 59
   this->codecCtx = this->formatCtx->streams[audioStream]->codec;
+#endif
 #ifndef _WIN32
 # pragma GCC diagnostic pop
 #endif
 
   // Find a decoder
-  this->codec = avcodec_find_decoder(codecCtx->codec_id);
+#if LIBAVFORMAT_VERSION_MAJOR >= 59
+  const AVCodec * local_codec = avcodec_find_decoder(this->formatCtx->streams[
+    this->audioStream]->codecpar->codec_id);
+  if (!local_codec)
+  {
+    gzerr << "Failed to find the codec" << std::endl;
+    return false;
+  }
+  this->codecCtx = avcodec_alloc_context3(local_codec);
+  if (!this->codecCtx)
+  {
+    gzerr << "Failed to allocate the codec context" << std::endl;
+    return false;
+  }
 
-  if (this->codec == NULL)
+  // Copy all relevant parameters from codepar to codecCtx
+  if (avcodec_parameters_to_context(this->codecCtx,
+    this->formatCtx->streams[this->audioStream]->codecpar) < 0)
+  {
+    gzerr << "Failed to copy codec parameters to decoder context"
+           << std::endl;
+    return false;
+  }
+
+  // This copy should be done by avcodec_parameters_to_context, but at least on
+  // conda-forge with 5.0.0 h594f047_1 this is not happening for some reason.
+  // As temporary workaround, we copy directly the data structures, taking the code from
+  // https://github.com/FFmpeg/FFmpeg/blob/n5.0/libavcodec/codec_par.c#L120
+  AVCodecParameters* par = this->formatCtx->streams[this->audioStream]->codecpar;
+  this->codecCtx->sample_fmt       = static_cast<AVSampleFormat>(par->format);
+  this->codecCtx->channel_layout   = par->channel_layout;
+  this->codecCtx->channels         = par->channels;
+  this->codecCtx->sample_rate      = par->sample_rate;
+  this->codecCtx->block_align      = par->block_align;
+  this->codecCtx->frame_size       = par->frame_size;
+  this->codecCtx->delay            =
+  this->codecCtx->initial_padding  = par->initial_padding;
+  this->codecCtx->trailing_padding = par->trailing_padding;
+  this->codecCtx->seek_preroll     = par->seek_preroll;
+
+  // It would be better to just define codec as const AVCodec *,
+  // but that is not done to avoid ABI problem. Anyhow, as codec
+  // it is a private attribute there should be no problem
+  this->codec = const_cast<AVCodec *>(local_codec);
+#else
+  this->codec = avcodec_find_decoder(codecCtx->codec_id);
+#endif
+
+  if (this->codec == nullptr)
   {
     gzerr << "Couldn't find codec for audio stream.\n";
     avformat_close_input(&this->formatCtx);
-    this->formatCtx = NULL;
+    this->formatCtx = nullptr;
 
     return false;
   }
 
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(56, 60, 100)
+  if (this->codec->capabilities & AV_CODEC_CAP_TRUNCATED)
+    this->codecCtx->flags |= AV_CODEC_FLAG_TRUNCATED;
+#else
   if (this->codec->capabilities & CODEC_CAP_TRUNCATED)
     this->codecCtx->flags |= CODEC_FLAG_TRUNCATED;
+#endif
 
   // Open codec
-  if (avcodec_open2(this->codecCtx, this->codec, NULL) < 0)
+  if (avcodec_open2(this->codecCtx, this->codec, nullptr) < 0)
   {
     gzerr << "Couldn't open audio codec.\n";
     avformat_close_input(&this->formatCtx);
-    this->formatCtx = NULL;
+    this->formatCtx = nullptr;
 
     return false;
   }
