@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,10 +23,6 @@
 using namespace gazebo;
 using namespace common;
 
-/// \brief Destination audio video frame
-/// TODO Do not merge forward. Declared here for gazebo7 ABI compatibility
-AVFrame *avFrameDst;
-
 /////////////////////////////////////////////////
 // #ifdef HAVE_FFMPEG
 // static void pgm_save(unsigned char *buf, int wrap, int xsize, int ysize,
@@ -43,17 +39,62 @@ AVFrame *avFrameDst;
 // }
 // #endif
 
+/////////////////////////////////////////////////
+#ifdef HAVE_FFMPEG
+int GazeboAVCodecDecodeHelper(AVCodecContext *_codecCtx,
+    AVFrame *_frame, int *_gotFrame, AVPacket *_packet)
+{
+#if LIBAVFORMAT_VERSION_MAJOR >= 59
+  // from https://blogs.gentoo.org/lu_zero/2016/03/29/new-avcodec-api/
+  int ret;
+
+  *_gotFrame = 0;
+
+  if (_packet)
+  {
+    ret = avcodec_send_packet(_codecCtx, _packet);
+    if (ret < 0)
+    {
+      return ret == AVERROR_EOF ? 0 : ret;
+    }
+  }
+
+  ret = avcodec_receive_frame(_codecCtx, _frame);
+  if (ret < 0 && ret != AVERROR(EAGAIN))
+  {
+    return ret;
+  }
+  if (ret >= 0)
+  {
+    *_gotFrame = 1;
+  }
+
+  // new API always consumes the whole packet
+  return _packet ? _packet->size : 0;
+#else
+  // this was deprecated in ffmpeg version 3.1
+  // github.com/FFmpeg/FFmpeg/commit/7fc329e2dd6226dfecaa4a1d7adf353bf2773726
+# ifndef _WIN32
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+# endif
+  return avcodec_decode_video2(_codecCtx, _frame, _gotFrame, _packet);
+# ifndef _WIN32
+#  pragma GCC diagnostic pop
+# endif
+#endif
+}
+#endif
+
 /////////////////////////////////////////////////
 Video::Video()
 {
-  this->formatCtx = NULL;
-  this->codecCtx = NULL;
-  this->swsCtx = NULL;
-  this->avFrame = NULL;
+  this->formatCtx = nullptr;
+  this->codecCtx = nullptr;
+  this->swsCtx = nullptr;
+  this->avFrame = nullptr;
   this->videoStream = -1;
-
-  this->pic = NULL;
-  avFrameDst = NULL;
+  this->avFrameDst = nullptr;
 }
 
 /////////////////////////////////////////////////
@@ -75,7 +116,7 @@ void Video::Cleanup()
   // Close the codec
   avcodec_close(this->codecCtx);
 
-  av_free(avFrameDst);
+  av_free(this->avFrameDst);
 #endif
 }
 
@@ -83,7 +124,7 @@ void Video::Cleanup()
 #ifdef HAVE_FFMPEG
 bool Video::Load(const std::string &_filename)
 {
-  AVCodec *codec = NULL;
+  const AVCodec *codec = nullptr;
   this->videoStream = -1;
 
   if (this->formatCtx || this->avFrame || this->codecCtx)
@@ -92,14 +133,15 @@ bool Video::Load(const std::string &_filename)
   this->avFrame = common::AVFrameAlloc();
 
   // Open video file
-  if (avformat_open_input(&this->formatCtx, _filename.c_str(), NULL, NULL) < 0)
+  if (avformat_open_input(&this->formatCtx, _filename.c_str(),
+        nullptr, nullptr) < 0)
   {
     gzerr << "Unable to read video file[" << _filename << "]\n";
     return false;
   }
 
   // Retrieve stream information
-  if (avformat_find_stream_info(this->formatCtx, NULL) < 0)
+  if (avformat_find_stream_info(this->formatCtx, nullptr) < 0)
   {
     gzerr << "Couldn't find stream information\n";
     return false;
@@ -108,6 +150,9 @@ bool Video::Load(const std::string &_filename)
   // Find the first video stream
   for (unsigned int i = 0; i < this->formatCtx->nb_streams; ++i)
   {
+#if LIBAVFORMAT_VERSION_MAJOR >= 59
+    if (this->formatCtx->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_VIDEO)
+#else
 #ifndef _WIN32
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wdeprecated-declarations"
@@ -115,6 +160,7 @@ bool Video::Load(const std::string &_filename)
     if (this->formatCtx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO)
 #ifndef _WIN32
 # pragma GCC diagnostic pop
+#endif
 #endif
     {
       this->videoStream = static_cast<int>(i);
@@ -129,6 +175,26 @@ bool Video::Load(const std::string &_filename)
   }
 
   // Get a pointer to the codec context for the video stream
+#if LIBAVFORMAT_VERSION_MAJOR >= 59
+  // AVCodecContext is not included in an AVStream as of ffmpeg 3.1
+  // allocate a codec context based on updated example
+  // github.com/FFmpeg/FFmpeg/commit/bba6a03b2816d805d44bce4f9701a71f7d3f8dad
+  this->codecCtx = avcodec_alloc_context3(codec);
+  if (!this->codecCtx)
+  {
+    gzerr << "Failed to allocate the codec context" << std::endl;
+    return false;
+  }
+
+  // Copy codec parameters from input stream to output codec context
+  if (avcodec_parameters_to_context(this->codecCtx,
+        this->formatCtx->streams[this->videoStream]->codecpar) < 0)
+  {
+    gzerr << "Failed to copy codec parameters to decoder context"
+           << std::endl;
+    return false;
+  }
+#else
 #ifndef _WIN32
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wdeprecated-declarations"
@@ -137,10 +203,12 @@ bool Video::Load(const std::string &_filename)
 #ifndef _WIN32
 # pragma GCC diagnostic pop
 #endif
+#endif
+
 
   // Find the decoder for the video stream
   codec = avcodec_find_decoder(this->codecCtx->codec_id);
-  if (codec == NULL)
+  if (codec == nullptr)
   {
     gzerr << "Codec not found\n";
     return false;
@@ -148,11 +216,16 @@ bool Video::Load(const std::string &_filename)
 
   // Inform the codec that we can handle truncated bitstreams -- i.e.,
   // bitstreams where frame boundaries can fall in the middle of packets
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(56, 60, 100)
+  if (codec->capabilities & AV_CODEC_CAP_TRUNCATED)
+    this->codecCtx->flags |= AV_CODEC_FLAG_TRUNCATED;
+#else
   if (codec->capabilities & CODEC_CAP_TRUNCATED)
     this->codecCtx->flags |= CODEC_FLAG_TRUNCATED;
+#endif
 
   // Open codec
-  if (avcodec_open2(this->codecCtx, codec, NULL) < 0)
+  if (avcodec_open2(this->codecCtx, codec, nullptr) < 0)
   {
     gzerr << "Could not open codec\n";
     return false;
@@ -165,19 +238,19 @@ bool Video::Load(const std::string &_filename)
       this->codecCtx->width,
       this->codecCtx->height,
       AV_PIX_FMT_RGB24,
-      SWS_BICUBIC, NULL, NULL, NULL);
+      SWS_BICUBIC, nullptr, nullptr, nullptr);
 
-  if (this->swsCtx == NULL)
+  if (this->swsCtx == nullptr)
   {
     gzerr << "Error while calling sws_getContext\n";
     return false;
   }
 
-  avFrameDst = common::AVFrameAlloc();
-  avFrameDst->format = this->codecCtx->pix_fmt;
-  avFrameDst->width = this->codecCtx->width;
-  avFrameDst->height = this->codecCtx->height;
-  av_image_alloc(avFrameDst->data, avFrameDst->linesize,
+  this->avFrameDst = common::AVFrameAlloc();
+  this->avFrameDst->format = this->codecCtx->pix_fmt;
+  this->avFrameDst->width = this->codecCtx->width;
+  this->avFrameDst->height = this->codecCtx->height;
+  av_image_alloc(this->avFrameDst->data, this->avFrameDst->linesize,
       this->codecCtx->width, this->codecCtx->height, this->codecCtx->pix_fmt,
       1);
 
@@ -227,15 +300,9 @@ bool Video::GetNextFrame(unsigned char **_buffer)
     while (tmpPacket.size > 0)
     {
       // sending data to libavcodec
-#ifndef _WIN32
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
-      int processedLength = avcodec_decode_video2(this->codecCtx, this->avFrame,
+      int processedLength = GazeboAVCodecDecodeHelper(this->codecCtx, this->avFrame,
           &frameAvailable, &tmpPacket);
-#ifndef _WIN32
-# pragma GCC diagnostic pop
-#endif
+
       if (processedLength < 0)
       {
         gzerr << "Error while processing the data\n";
@@ -249,10 +316,10 @@ bool Video::GetNextFrame(unsigned char **_buffer)
       if (frameAvailable)
       {
         sws_scale(swsCtx, this->avFrame->data, this->avFrame->linesize, 0,
-            this->codecCtx->height, avFrameDst->data,
-            avFrameDst->linesize);
+            this->codecCtx->height, this->avFrameDst->data,
+            this->avFrameDst->linesize);
 
-        memcpy(*_buffer, avFrameDst->data[0],
+        memcpy(*_buffer, this->avFrameDst->data[0],
             this->codecCtx->height * (this->codecCtx->width*3));
 
         // Debug:
