@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2016 Open Source Robotics Foundation
+ * Copyright (C) 2015 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -48,7 +48,8 @@ void GraphView::resizeEvent(QResizeEvent */*_event*/)
 /////////////////////////////////////////////////
 void GraphView::contextMenuEvent(QContextMenuEvent *_event)
 {
-  QGraphicsItem *item = this->scene()->itemAt(this->mapToScene(_event->pos()));
+  QGraphicsItem *item = this->scene()->itemAt(
+      this->mapToScene(_event->pos()), this->transform());
   if (item)
   {
     QString itemData = item->data(0).toString();
@@ -68,8 +69,9 @@ void GraphView::contextMenuEvent(QContextMenuEvent *_event)
 void GraphView::wheelEvent(QWheelEvent *_event)
 {
   int wheelIncr = 120;
-  int sign = (_event->delta() > 0) ? 1 : -1;
-  int delta = std::max(std::abs(_event->delta()), wheelIncr) * sign;
+  int rawDelta = _event->angleDelta().y();
+  int sign = (rawDelta > 0) ? 1 : -1;
+  int delta = std::max(std::abs(rawDelta), wheelIncr) * sign;
   int numSteps = delta / wheelIncr;
 
   QMatrix mat = matrix();
@@ -112,7 +114,8 @@ void GraphView::mouseReleaseEvent(QMouseEvent *_event)
     return;
   }
 
-  QGraphicsItem *item = this->scene()->itemAt(this->mapToScene(_event->pos()));
+  QGraphicsItem *item = this->scene()->itemAt(
+      this->mapToScene(_event->pos()), this->transform());
   this->lastClickedItem = item;
   if (item)
   {
@@ -165,7 +168,8 @@ void GraphView::keyPressEvent(QKeyEvent *_event)
 /////////////////////////////////////////////////
 void GraphView::mouseDoubleClickEvent(QMouseEvent *_event)
 {
-  QGraphicsItem *item = this->scene()->itemAt(this->mapToScene(_event->pos()));
+  QGraphicsItem *item = this->scene()->itemAt(
+      this->mapToScene(_event->pos()), this->transform());
   if (item)
   {
     QString itemData = item->data(0).toString();
