@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Open Source Robotics Foundation
+ * Copyright (C) 2014 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
  *
 */
 
-#include <boost/bind.hpp>
+#include <boost/bind/bind.hpp>
 #include <string>
 
 #include "gazebo/gazebo_config.h"
@@ -64,20 +64,6 @@ ModelEditor::ModelEditor(MainWindow *_mainWindow)
 
   GZ_ASSERT(this->tabWidget != NULL, "Editor tab widget is NULL");
 
-  rendering::CameraPtr camera = boost::dynamic_pointer_cast<rendering::Camera>(
-      gui::get_active_camera());
-  if (camera)
-  {
-    this->dataPtr->materialSwitcher.reset(new EditorMaterialSwitcher(camera));
-  }
-  else
-  {
-    gzerr << "User camera is NULL. "
-        << "Non-editable models will keep their original material"
-        << std::endl;
-  }
-
-
   this->dataPtr->schematicViewAct = NULL;
   this->dataPtr->svWidget = NULL;
 #ifdef HAVE_GRAPHVIZ
@@ -286,6 +272,7 @@ ModelEditor::ModelEditor(MainWindow *_mainWindow)
   connect(this->dataPtr->modelPalette->ModelCreator()->JointMaker(),
       SIGNAL(JointAdded()), this, SLOT(OnJointAdded()));
 
+  using namespace boost::placeholders;
   this->dataPtr->connections.push_back(
       gui::Events::ConnectCreateEntity(
         boost::bind(&ModelEditor::OnCreateEntity, this, _1, _2)));
@@ -387,6 +374,15 @@ void ModelEditor::CreateMenus()
   }
   windowMenu->addAction(this->dataPtr->showToolbarsAct);
   windowMenu->addAction(this->dataPtr->fullScreenAct);
+
+  // OSX:
+  // There is a problem on osx with the qt5 menubar being out of focus when
+  // the application is launched from a terminal, so prevent using a native
+  // menubar for now.
+  //
+  // Ubuntu Xenial + Unity:
+  // The native menubar is not registering shortcuts (issue #2134)
+  this->dataPtr->menuBar->setNativeMenuBar(false);
 }
 
 /////////////////////////////////////////////////
@@ -515,7 +511,27 @@ void ModelEditor::OnFinish()
 void ModelEditor::ToggleMaterialScheme()
 {
   if (this->dataPtr->active)
+  {
+    if (!this->dataPtr->materialSwitcher)
+    {
+      rendering::CameraPtr camera =
+          boost::dynamic_pointer_cast<rendering::Camera>(
+          gui::get_active_camera());
+      if (camera)
+      {
+        this->dataPtr->materialSwitcher.reset(
+            new EditorMaterialSwitcher(camera));
+      }
+      else
+      {
+        gzerr << "User camera is NULL. "
+            << "Non-editable models will keep their original material"
+            << std::endl;
+      }
+    }
+
     this->dataPtr->materialSwitcher->SetMaterialScheme("ModelEditor");
+  }
   else
     this->dataPtr->materialSwitcher->SetMaterialScheme("");
 }
