@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@
  * Date: 15 May 2009
  */
 
-#include <boost/bind.hpp>
+#include <boost/bind/bind.hpp>
 #include <string>
 
 #include "gazebo/common/Assert.hh"
@@ -38,9 +38,9 @@ using namespace physics;
 BulletJoint::BulletJoint(BasePtr _parent)
   : Joint(_parent)
 {
-  this->constraint = NULL;
-  this->bulletWorld = NULL;
-  this->feedback = NULL;
+  this->constraint = nullptr;
+  this->bulletWorld = nullptr;
+  this->feedback = nullptr;
   this->stiffnessDampingInitialized = false;
   this->forceApplied[0] = 0;
   this->forceApplied[1] = 0;
@@ -60,12 +60,12 @@ void BulletJoint::Fini()
     this->bulletWorld->removeConstraint(this->constraint);
     delete this->constraint;
   }
-  this->constraint = NULL;
-  this->bulletWorld = NULL;
+  this->constraint = nullptr;
+  this->bulletWorld = nullptr;
 
   if (this->feedback)
     delete this->feedback;
-  this->feedback = NULL;
+  this->feedback = nullptr;
 
   Joint::Fini();
 }
@@ -123,7 +123,7 @@ LinkPtr BulletJoint::GetJointLink(unsigned int _index) const
 {
   LinkPtr result;
 
-  if (this->constraint == NULL)
+  if (this->constraint == nullptr)
     gzthrow("Attach bodies to the joint first");
 
   if (_index == 0 || _index == 1)
@@ -164,7 +164,7 @@ void BulletJoint::Detach()
   if (this->constraint && this->bulletWorld)
     this->bulletWorld->removeConstraint(this->constraint);
   delete this->constraint;
-  this->constraint = NULL;
+  this->constraint = nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -184,13 +184,13 @@ void BulletJoint::CacheForceTorque()
   // caching force torque for the joint
   // if cached, GetForceTorque should use this value
   // this->wrench
-  this->wrench.body2Force = BulletTypes::ConvertVector3(
+  this->wrench.body2Force = BulletTypes::ConvertVector3Ign(
                       this->feedback->m_appliedForceBodyA);
-  this->wrench.body2Torque = BulletTypes::ConvertVector3(
+  this->wrench.body2Torque = BulletTypes::ConvertVector3Ign(
                       this->feedback->m_appliedTorqueBodyA);
-  this->wrench.body1Force = BulletTypes::ConvertVector3(
+  this->wrench.body1Force = BulletTypes::ConvertVector3Ign(
                       this->feedback->m_appliedForceBodyB);
-  this->wrench.body1Torque = BulletTypes::ConvertVector3(
+  this->wrench.body1Torque = BulletTypes::ConvertVector3Ign(
                       this->feedback->m_appliedTorqueBodyB);
   // gzerr << "   " << this->GetName()
   //       << " : " << this->wrench.body1Force
@@ -204,12 +204,12 @@ void BulletJoint::CacheForceTorque()
   if (this->HasType(physics::Base::HINGE_JOINT))
   {
     // rotate force into child link frame
-    // GetLocalAxis is the axis specified in parent link frame!!!
+    // LocalAxis is the axis specified in parent link frame!!!
     wrenchAppliedWorld.body2Torque =
-      this->GetForce(0u) * this->GetLocalAxis(0u);
+      this->GetForce(0u) * this->LocalAxis(0u);
 
     // gzerr << "body2Torque [" << wrenchAppliedWorld.body2Torque
-    //       << "] axis [" << this->GetLocalAxis(0u)
+    //       << "] axis [" << this->LocalAxis(0u)
     //       << "]\n";
 
     wrenchAppliedWorld.body1Torque = -wrenchAppliedWorld.body2Torque;
@@ -218,7 +218,7 @@ void BulletJoint::CacheForceTorque()
   {
     // rotate force into child link frame
     wrenchAppliedWorld.body2Force =
-      this->GetForce(0u) * this->GetLocalAxis(0u);
+      this->GetForce(0u) * this->LocalAxis(0u);
     wrenchAppliedWorld.body1Force = -wrenchAppliedWorld.body2Force;
   }
   else
@@ -232,20 +232,23 @@ void BulletJoint::CacheForceTorque()
   // convert wrench from child cg location to child link frame
   if (this->childLink)
   {
-    math::Pose childPose = this->childLink->GetWorldPose();
+    ignition::math::Pose3d childPose = this->childLink->WorldPose();
 
     // convert torque from about child CG to joint anchor location
     // cg position specified in child link frame
-    math::Pose cgPose = this->childLink->GetInertial()->GetPose();
+    ignition::math::Pose3d cgPose =
+      this->childLink->GetInertial()->Pose();
 
     // anchorPose location of joint in child frame
     // childMomentArm: from child CG to joint location in child link frame
     // moment arm rotated into world frame (given feedback is in world frame)
-    math::Vector3 childMomentArm = childPose.rot.RotateVector(
-      (this->anchorPose - math::Pose(cgPose.pos, math::Quaternion())).pos);
+    ignition::math::Vector3d childMomentArm = childPose.Rot().RotateVector(
+      (this->anchorPose -
+       ignition::math::Pose3d(cgPose.Pos(),
+         ignition::math::Quaterniond::Identity)).Pos());
 
     // gzerr << "anchor [" << anchorPose
-    //       << "] iarm[" << this->childLink->GetInertial()->GetPose().pos
+    //       << "] iarm[" << this->childLink->GetInertial()->GetPose().Pos()
     //       << "] childMomentArm[" << childMomentArm
     //       << "] f1[" << this->wrench.body2Force
     //       << "] t1[" << this->wrench.body2Torque
@@ -255,45 +258,43 @@ void BulletJoint::CacheForceTorque()
     this->wrench.body2Torque += this->wrench.body2Force.Cross(childMomentArm);
 
     // rotate resulting body2Force in world frame into link frame
-    this->wrench.body2Force = childPose.rot.RotateVectorReverse(
+    this->wrench.body2Force = childPose.Rot().RotateVectorReverse(
       -this->wrench.body2Force);
 
     // rotate resulting body2Torque in world frame into link frame
-    this->wrench.body2Torque = childPose.rot.RotateVectorReverse(
+    this->wrench.body2Torque = childPose.Rot().RotateVectorReverse(
       -this->wrench.body2Torque);
   }
 
   // convert torque from about parent CG to joint anchor location
   if (this->parentLink)
   {
-    // get child pose, or it's the inertial world if childLink is NULL
-    math::Pose childPose;
+    // get child pose, or it's the inertial world if childLink is nullptr
+    ignition::math::Pose3d childPose;
     if (this->childLink)
-      childPose = this->childLink->GetWorldPose();
+      childPose = this->childLink->WorldPose();
 
-    math::Pose parentPose = this->parentLink->GetWorldPose();
+    ignition::math::Pose3d parentPose = this->parentLink->WorldPose();
 
     // if parent link exists, convert torque from about parent
     // CG to joint anchor location
 
     // parent cg specified in parent link frame
-    math::Pose cgPose = this->parentLink->GetInertial()->GetPose();
+    ignition::math::Pose3d cgPose =
+      this->parentLink->GetInertial()->Pose();
 
     // get parent CG pose in child link frame
-    math::Pose parentCGInChildLink =
-      math::Pose(cgPose.pos, math::Quaternion()) - (childPose - parentPose);
-
-    // anchor location in parent CG frame
-    // this is the moment arm, but it's in parent CG frame, we need
-    // to convert it into world frame
-    math::Pose anchorInParendCGFrame = this->anchorPose - parentCGInChildLink;
+    ignition::math::Pose3d parentCGInChildLink =
+      ignition::math::Pose3d(cgPose.Pos(),
+          ignition::math::Quaterniond::Identity) - (childPose - parentPose);
 
     // paretnCGFrame in world frame
-    math::Pose parentCGInWorld = cgPose + parentPose;
+    ignition::math::Pose3d parentCGInWorld = cgPose + parentPose;
 
     // rotate momeent arms into world frame
-    math::Vector3 parentMomentArm = parentCGInWorld.rot.RotateVector(
-      (this->anchorPose - parentCGInChildLink).pos);
+    ignition::math::Vector3d parentMomentArm =
+      parentCGInWorld.Rot().RotateVector(
+          (this->anchorPose - parentCGInChildLink).Pos());
 
     // gzerr << "anchor [" << this->anchorPose
     //       << "] pcginc[" << parentCGInChildLink
@@ -305,14 +306,15 @@ void BulletJoint::CacheForceTorque()
     //       << "] fxp[" << this->wrench.body1Force.Cross(parentMomentArm)
     //       << "]\n";
 
-    this->wrench.body1Torque += this->wrench.body1Force.Cross(parentMomentArm);
+    this->wrench.body1Torque += this->wrench.body1Force.Cross(
+        parentMomentArm);
 
     // rotate resulting body1Force in world frame into link frame
-    this->wrench.body1Force = parentPose.rot.RotateVectorReverse(
+    this->wrench.body1Force = parentPose.Rot().RotateVectorReverse(
       -this->wrench.body1Force);
 
     // rotate resulting body1Torque in world frame into link frame
-    this->wrench.body1Torque = parentPose.rot.RotateVectorReverse(
+    this->wrench.body1Torque = parentPose.Rot().RotateVectorReverse(
       -this->wrench.body1Torque);
 
     if (!this->childLink)
@@ -323,12 +325,12 @@ void BulletJoint::CacheForceTorque()
 
       // force/torque are in parent link frame, transform them into
       // child link(world) frame.
-      math::Pose parentToWorldTransform = this->parentLink->GetWorldPose();
+      auto parentToWorldTransform = this->parentLink->WorldPose();
       this->wrench.body1Force =
-        parentToWorldTransform.rot.RotateVector(
+        parentToWorldTransform.Rot().RotateVector(
         this->wrench.body1Force);
       this->wrench.body1Torque =
-        parentToWorldTransform.rot.RotateVector(
+        parentToWorldTransform.Rot().RotateVector(
         this->wrench.body1Torque);
     }
   }
@@ -348,12 +350,12 @@ void BulletJoint::CacheForceTorque()
 
       // force/torque are in child link frame, transform them into
       // parent link frame.  Here, parent link is world, so zero transform.
-      math::Pose childToWorldTransform = this->childLink->GetWorldPose();
+      auto childToWorldTransform = this->childLink->WorldPose();
       this->wrench.body1Force =
-        childToWorldTransform.rot.RotateVector(
+        childToWorldTransform.Rot().RotateVector(
         this->wrench.body1Force);
       this->wrench.body1Torque =
-        childToWorldTransform.rot.RotateVector(
+        childToWorldTransform.Rot().RotateVector(
         this->wrench.body1Torque);
     }
   }
@@ -363,7 +365,7 @@ void BulletJoint::CacheForceTorque()
 //////////////////////////////////////////////////
 JointWrench BulletJoint::GetForceTorque(unsigned int /*_index*/)
 {
-  GZ_ASSERT(this->constraint != NULL, "constraint should be valid");
+  GZ_ASSERT(this->constraint != nullptr, "constraint should be valid");
   return this->wrench;
 }
 
@@ -372,7 +374,7 @@ void BulletJoint::SetupJointFeedback()
 {
   if (this->provideFeedback)
   {
-    if (this->feedback == NULL)
+    if (this->feedback == nullptr)
     {
       this->feedback = new btJointFeedback;
       this->feedback->m_appliedForceBodyA = btVector3(0, 0, 0);
@@ -391,7 +393,7 @@ void BulletJoint::SetupJointFeedback()
 //////////////////////////////////////////////////
 void BulletJoint::SetDamping(unsigned int _index, double _damping)
 {
-  if (_index < this->GetAngleCount())
+  if (_index < this->DOF())
   {
     this->SetStiffnessDamping(_index, this->stiffnessCoefficient[_index],
       _damping);
@@ -399,16 +401,16 @@ void BulletJoint::SetDamping(unsigned int _index, double _damping)
   else
   {
      gzerr << "BulletJoint::SetDamping: index[" << _index
-           << "] is out of bounds (GetAngleCount() = "
-           << this->GetAngleCount() << ").\n";
+           << "] is out of bounds (DOF() = "
+           << this->DOF() << ").\n";
      return;
   }
 }
 
 //////////////////////////////////////////////////
-void BulletJoint::SetStiffness(unsigned int _index, double _stiffness)
+void BulletJoint::SetStiffness(unsigned int _index, const double _stiffness)
 {
-  if (_index < this->GetAngleCount())
+  if (_index < this->DOF())
   {
     this->SetStiffnessDamping(_index, _stiffness,
       this->dissipationCoefficient[_index]);
@@ -416,8 +418,8 @@ void BulletJoint::SetStiffness(unsigned int _index, double _stiffness)
   else
   {
      gzerr << "BulletJoint::SetStiffness: index[" << _index
-           << "] is out of bounds (GetAngleCount() = "
-           << this->GetAngleCount() << ").\n";
+           << "] is out of bounds (DOF() = "
+           << this->DOF() << ").\n";
      return;
   }
 }
@@ -426,7 +428,7 @@ void BulletJoint::SetStiffness(unsigned int _index, double _stiffness)
 void BulletJoint::SetStiffnessDamping(unsigned int _index,
   double _stiffness, double _damping, double _reference)
 {
-  if (_index < this->GetAngleCount())
+  if (_index < this->DOF())
   {
     this->stiffnessCoefficient[_index] = _stiffness;
     this->dissipationCoefficient[_index] = _damping;
@@ -476,12 +478,12 @@ void BulletJoint::SaveForce(unsigned int _index, double _force)
 {
   // this bit of code actually doesn't do anything physical,
   // it simply records the forces commanded inside forceApplied.
-  if (_index < this->GetAngleCount())
+  if (_index < this->DOF())
   {
-    if (this->forceAppliedTime < this->GetWorld()->GetSimTime())
+    if (this->forceAppliedTime < this->GetWorld()->SimTime())
     {
       // reset forces if time step is new
-      this->forceAppliedTime = this->GetWorld()->GetSimTime();
+      this->forceAppliedTime = this->GetWorld()->SimTime();
       this->forceApplied[0] = this->forceApplied[1] = 0;
     }
 
@@ -496,7 +498,7 @@ void BulletJoint::SaveForce(unsigned int _index, double _force)
 //////////////////////////////////////////////////
 double BulletJoint::GetForce(unsigned int _index)
 {
-  if (_index < this->GetAngleCount())
+  if (_index < this->DOF())
   {
     return this->forceApplied[_index];
   }
@@ -511,7 +513,7 @@ double BulletJoint::GetForce(unsigned int _index)
 //////////////////////////////////////////////////
 void BulletJoint::ApplyStiffnessDamping()
 {
-  for (unsigned int i = 0; i < this->GetAngleCount(); ++i)
+  for (unsigned int i = 0; i < this->DOF(); ++i)
   {
     // Take absolute value of dissipationCoefficient, since negative values of
     // dissipationCoefficient are used for adaptive damping to
@@ -520,7 +522,7 @@ void BulletJoint::ApplyStiffnessDamping()
       * this->GetVelocity(i);
 
     double springForce = this->stiffnessCoefficient[i]
-      * (this->springReferencePosition[i] - this->GetAngle(i).Radian());
+      * (this->springReferencePosition[i] - this->Position(i));
 
     // do not change forceApplied if setting internal damping forces
     this->SetForceImpl(i, dampingForce + springForce);
@@ -530,31 +532,34 @@ void BulletJoint::ApplyStiffnessDamping()
 }
 
 //////////////////////////////////////////////////
-void BulletJoint::SetAnchor(unsigned int /*_index*/,
-    const gazebo::math::Vector3 & /*_anchor*/)
+void BulletJoint::SetAnchor(const unsigned int /*_index*/,
+    const ignition::math::Vector3d & /*_anchor*/)
 {
   // nothing to do here for bullet.
 }
 
 //////////////////////////////////////////////////
-math::Vector3 BulletJoint::GetAnchor(unsigned int /*_index*/) const
+ignition::math::Vector3d BulletJoint::Anchor(
+    const unsigned int /*_index*/) const
 {
   gzerr << "Not implement in Bullet\n";
-  return math::Vector3();
+  return ignition::math::Vector3d::Zero;
 }
 
 //////////////////////////////////////////////////
-math::Vector3 BulletJoint::GetLinkForce(unsigned int /*_index*/) const
+ignition::math::Vector3d BulletJoint::LinkForce(
+          const unsigned int /*_index*/) const
 {
   gzerr << "Not implement in Bullet\n";
-  return math::Vector3();
+  return ignition::math::Vector3d::Zero;
 }
 
 //////////////////////////////////////////////////
-math::Vector3 BulletJoint::GetLinkTorque(unsigned int /*_index*/) const
+ignition::math::Vector3d BulletJoint::LinkTorque(
+          const unsigned int /*_index*/) const
 {
   gzerr << "Not implement in Bullet\n";
-  return math::Vector3();
+  return ignition::math::Vector3d::Zero;
 }
 
 //////////////////////////////////////////////////
@@ -574,19 +579,72 @@ double BulletJoint::GetParam(const std::string &_key,
 }
 
 //////////////////////////////////////////////////
-math::Angle BulletJoint::GetHighStop(unsigned int _index)
+bool BulletJoint::SetPosition(const unsigned int _index, const double _position,
+                              const bool _preserveWorldVelocity)
 {
-  return this->GetUpperLimit(_index);
-}
+  // The code inside this ifdef is only relevant for versions of bullet greater
+  // than 2.82. Any versions earlier than that will be broken no matter what.
+#ifdef LIBBULLET_VERSION_GT_282
+  // The following code fixes issue 2430 without breaking ABI. The key is to
+  // take relatively small steps towards a target position so that the
+  // accumulated angle doesn't bug out and reset itself from receiving too large
+  // of a change all at once.
+  if (this->HasType(Base::HINGE_JOINT))
+  {
+    double currentAngle = this->Position(0);
 
-//////////////////////////////////////////////////
-math::Angle BulletJoint::GetLowStop(unsigned int _index)
-{
-  return this->GetLowerLimit(_index);
-}
+    const double maxSupportedAngle = 1e12;
+    if (   std::abs(currentAngle) > maxSupportedAngle
+        || std::abs(_position) > maxSupportedAngle)
+    {
+      // If either the current or the target angle are absurdly large, we will
+      // get permanently stuck in the while-loop below, because trying to
+      // increment currentAngle by delta will (eventually) not be able to modify
+      // its floating point value.
+      //
+      // We put this check here to make sure that we can't get permanently stuck
+      // in the loop.
+      gzerr << "For Bullet hinge joints, the function Joint::SetPosition(~) "
+            << "does not support positions larger than [" << maxSupportedAngle
+            << "] radians. Your joint currently has an angle of ["
+            << currentAngle << "] radians, and you are requesting an angle of ["
+            << _position << "] radians.\n";
 
-//////////////////////////////////////////////////
-bool BulletJoint::SetPosition(unsigned int _index, double _position)
-{
-  return Joint::SetPositionMaximal(_index, _position);
+      // We can still call Joint::SetPositionMaximal in the normal way to reset
+      // this joint angle. The end result should put this joint angle within
+      // the range of [-pi, pi]. That should at least bring it back to a sane
+      // state, even if it's not exactly what the user asked for.
+      Joint::SetPositionMaximal(_index, _position, _preserveWorldVelocity);
+
+      return false;
+    }
+
+    // Based on the source code of Bullet, the largest position change that
+    // shouldn't bug out is 0.3 radians, so we keep our changes a little bit
+    // below that.
+    const double delta = 0.25;
+
+    do
+    {
+      if (std::abs(_position - currentAngle) > delta)
+      {
+        currentAngle += _position > currentAngle? delta : -delta;
+      }
+      else
+      {
+        currentAngle = _position;
+      }
+
+      if (!Joint::SetPositionMaximal(_index, currentAngle,
+                                     _preserveWorldVelocity))
+      {
+        return false;
+      }
+    } while ( std::abs(currentAngle - _position) > 1e-16);
+
+    return true;
+  }
+#endif
+
+  return Joint::SetPositionMaximal(_index, _position, _preserveWorldVelocity);
 }
