@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,12 +15,6 @@
  *
 */
 
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-#endif
-
 #include <tbb/parallel_for.h>
 #include <tbb/blocked_range.h>
 
@@ -32,12 +26,16 @@
 #include <utility>
 #include <vector>
 
+#include <ignition/math/Rand.hh>
+#include <ignition/math/Vector3.hh>
+#include <ignition/common/Profiler.hh>
+
+#include <sdf/Param.hh>
+
 #include "gazebo/util/Diagnostics.hh"
 #include "gazebo/common/Assert.hh"
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/Exception.hh"
-#include "gazebo/math/Vector3.hh"
-#include "gazebo/math/Rand.hh"
 #include "gazebo/common/Time.hh"
 #include "gazebo/common/Timer.hh"
 
@@ -138,7 +136,7 @@ extern "C" void dMessageQuiet(int, const char *, va_list)
 ODEPhysics::ODEPhysics(WorldPtr _world)
     : PhysicsEngine(_world), dataPtr(new ODEPhysicsPrivate)
 {
-  this->dataPtr->physicsStepFunc = NULL;
+  this->dataPtr->physicsStepFunc = nullptr;
   this->dataPtr->maxContacts = 0;
 
   // Collision detection init
@@ -157,7 +155,7 @@ ODEPhysics::ODEPhysics(WorldPtr _world)
 
   // Set random seed for physics engine based on gazebo's random seed.
   // Note: this was moved from physics::PhysicsEngine constructor.
-  this->SetSeed(math::Rand::GetSeed());
+  this->SetSeed(ignition::math::Rand::Seed());
 }
 
 //////////////////////////////////////////////////
@@ -166,7 +164,7 @@ ODEPhysics::~ODEPhysics()
   this->Fini();
 
   delete this->dataPtr;
-  this->dataPtr = NULL;
+  this->dataPtr = nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -236,7 +234,7 @@ void ODEPhysics::Load(sdf::ElementPtr _sdf)
 
   // Set the physics update function
   this->SetStepType(this->dataPtr->stepType);
-  if (this->dataPtr->physicsStepFunc == NULL)
+  if (this->dataPtr->physicsStepFunc == nullptr)
     gzthrow(std::string("Invalid step type[") + this->dataPtr->stepType);
 }
 
@@ -267,7 +265,7 @@ void ODEPhysics::OnRequest(ConstRequestPtr &_msg)
     }
     physicsMsg.set_precon_iters(this->GetSORPGSPreconIters());
     physicsMsg.set_iters(this->GetSORPGSIters());
-    physicsMsg.set_enable_physics(this->world->GetEnablePhysicsEngine());
+    physicsMsg.set_enable_physics(this->world->PhysicsEnabled());
     physicsMsg.set_sor(this->GetSORPGSW());
     physicsMsg.set_cfm(this->GetWorldCFM());
     physicsMsg.set_erp(this->GetWorldERP());
@@ -276,9 +274,9 @@ void ODEPhysics::OnRequest(ConstRequestPtr &_msg)
     physicsMsg.set_contact_surface_layer(
       this->GetContactSurfaceLayer());
     physicsMsg.mutable_gravity()->CopyFrom(
-      msgs::Convert(this->GetGravity().Ign()));
+      msgs::Convert(this->world->Gravity()));
     physicsMsg.mutable_magnetic_field()->CopyFrom(
-      msgs::Convert(this->MagneticField()));
+      msgs::Convert(this->world->MagneticField()));
     physicsMsg.set_real_time_update_rate(this->realTimeUpdateRate);
     physicsMsg.set_real_time_factor(this->targetRealTimeFactor);
     physicsMsg.set_max_step_size(this->maxStepSize);
@@ -319,7 +317,7 @@ void ODEPhysics::OnPhysicsMsg(ConstPhysicsPtr &_msg)
     this->SetWorldERP(_msg->erp());
 
   if (_msg->has_enable_physics())
-    this->world->EnablePhysicsEngine(_msg->enable_physics());
+    this->world->SetPhysicsEnabled(_msg->enable_physics());
 
   if (_msg->has_contact_max_correcting_vel())
     this->SetContactMaxCorrectingVel(_msg->contact_max_correcting_vel());
@@ -357,6 +355,7 @@ void ODEPhysics::Init()
 //////////////////////////////////////////////////
 void ODEPhysics::InitForThread()
 {
+  IGN_PROFILE_THREAD_NAME("ODEPhysics");
   dAllocateODEDataForThread(dAllocateMaskAll);
 }
 
@@ -364,6 +363,8 @@ void ODEPhysics::InitForThread()
 void ODEPhysics::UpdateCollision()
 {
   DIAG_TIMER_START("ODEPhysics::UpdateCollision");
+  IGN_PROFILE("ODEPhysics:UpdateCollision");
+  IGN_PROFILE_BEGIN("dSpaceCollide");
 
   boost::recursive_mutex::scoped_lock lock(*this->physicsUpdateMutex);
   dJointGroupEmpty(this->dataPtr->contactGroup);
@@ -379,7 +380,9 @@ void ODEPhysics::UpdateCollision()
   // Do collision detection; this will add contacts to the contact group
   dSpaceCollide(this->dataPtr->spaceId, this, CollisionCallback);
   DIAG_TIMER_LAP("ODEPhysics::UpdateCollision", "dSpaceCollide");
+  IGN_PROFILE_END();
 
+  IGN_PROFILE_BEGIN("collideShapes");
   // Generate non-trimesh collisions.
   for (i = 0; i < this->dataPtr->collidersCount; ++i)
   {
@@ -387,7 +390,10 @@ void ODEPhysics::UpdateCollision()
         this->dataPtr->colliders[i].second, this->dataPtr->contactCollisions);
   }
   DIAG_TIMER_LAP("ODEPhysics::UpdateCollision", "collideShapes");
+  IGN_PROFILE_END();
 
+
+  IGN_PROFILE_BEGIN("collideTrimeshes");
   // Generate trimesh collision.
   // This must happen in this thread sequentially
   for (i = 0; i < this->dataPtr->trimeshCollidersCount; ++i)
@@ -396,7 +402,8 @@ void ODEPhysics::UpdateCollision()
     ODECollision *collision2 = this->dataPtr->trimeshColliders[i].second;
     this->Collide(collision1, collision2, this->dataPtr->contactCollisions);
   }
-  DIAG_TIMER_LAP("ODEPhysics::UpdateCollision", "collideTrimeshes");
+  DIAG_TIMER_LAP("UpdateCollision", "collideTrimeshes");
+  IGN_PROFILE_END();
 
   DIAG_TIMER_STOP("ODEPhysics::UpdateCollision");
 }
@@ -405,6 +412,7 @@ void ODEPhysics::UpdateCollision()
 void ODEPhysics::UpdatePhysics()
 {
   DIAG_TIMER_START("ODEPhysics::UpdatePhysics");
+  IGN_PROFILE("ODEPhysics:UpdatePhysics");
 
   // need to lock, otherwise might conflict with world resetting
   {
@@ -414,7 +422,7 @@ void ODEPhysics::UpdatePhysics()
     (*(this->dataPtr->physicsStepFunc))
       (this->dataPtr->worldId, this->maxStepSize);
 
-    math::Vector3 f1, f2, t1, t2;
+    ignition::math::Vector3d f1, f2, t1, t2;
 
     // Set the joint contact feedback for each contact.
     for (unsigned int i = 0; i < this->dataPtr->jointFeedbackIndex; ++i)
@@ -423,8 +431,8 @@ void ODEPhysics::UpdatePhysics()
       Collision *col1 = contactFeedback->collision1;
       Collision *col2 = contactFeedback->collision2;
 
-      GZ_ASSERT(col1 != NULL, "Collision 1 is NULL");
-      GZ_ASSERT(col2 != NULL, "Collision 2 is NULL");
+      GZ_ASSERT(col1 != nullptr, "Collision 1 is null");
+      GZ_ASSERT(col2 != nullptr, "Collision 2 is null");
 
       for (int j = 0; j < this->dataPtr->jointFeedbacks[i]->count; ++j)
       {
@@ -436,13 +444,13 @@ void ODEPhysics::UpdatePhysics()
 
         // set force torque in link frame
         this->dataPtr->jointFeedbacks[i]->contact->wrench[j].body1Force =
-             col1->GetLink()->GetWorldPose().rot.RotateVectorReverse(f1);
+             col1->GetLink()->WorldPose().Rot().RotateVectorReverse(f1);
         this->dataPtr->jointFeedbacks[i]->contact->wrench[j].body2Force =
-             col2->GetLink()->GetWorldPose().rot.RotateVectorReverse(f2);
+             col2->GetLink()->WorldPose().Rot().RotateVectorReverse(f2);
         this->dataPtr->jointFeedbacks[i]->contact->wrench[j].body1Torque =
-             col1->GetLink()->GetWorldPose().rot.RotateVectorReverse(t1);
+             col1->GetLink()->WorldPose().Rot().RotateVectorReverse(t1);
         this->dataPtr->jointFeedbacks[i]->contact->wrench[j].body2Torque =
-             col2->GetLink()->GetWorldPose().rot.RotateVectorReverse(t2);
+             col2->GetLink()->WorldPose().Rot().RotateVectorReverse(t2);
       }
     }
   }
@@ -455,7 +463,9 @@ void ODEPhysics::Fini()
 {
   dCloseODE();
 
-  dJointGroupDestroy(this->dataPtr->contactGroup);
+  if (this->dataPtr->contactGroup)
+    dJointGroupDestroy(this->dataPtr->contactGroup);
+  this->dataPtr->contactGroup = nullptr;
 
   // Delete all the joint feedbacks.
   for (auto iter = this->dataPtr->jointFeedbacks.begin();
@@ -473,9 +483,9 @@ void ODEPhysics::Fini()
 
   if (this->dataPtr->worldId)
     dWorldDestroy(this->dataPtr->worldId);
-  this->dataPtr->worldId = NULL;
+  this->dataPtr->worldId = nullptr;
 
-  this->dataPtr->spaceId = NULL;
+  this->dataPtr->spaceId = nullptr;
 
   PhysicsEngine::Fini();
 }
@@ -491,7 +501,7 @@ void ODEPhysics::Reset()
 //////////////////////////////////////////////////
 LinkPtr ODEPhysics::CreateLink(ModelPtr _parent)
 {
-  if (_parent == NULL)
+  if (_parent == nullptr)
     gzthrow("Link must have a parent\n");
 
   std::map<std::string, dSpaceID>::iterator iter;
@@ -539,7 +549,12 @@ ShapePtr ODEPhysics::CreateShape(const std::string &_type,
   else if (_type == "polyline")
     shape.reset(new ODEPolylineShape(collision));
   else if (_type == "multiray")
-    shape.reset(new ODEMultiRayShape(collision));
+  {
+    if (_collision)
+      shape.reset(new ODEMultiRayShape(collision));
+    else
+      shape.reset(new ODEMultiRayShape(this->world->Physics()));
+  }
   else if (_type == "mesh" || _type == "trimesh")
     shape.reset(new ODEMeshShape(collision));
   else if (_type == "heightmap")
@@ -547,10 +562,12 @@ ShapePtr ODEPhysics::CreateShape(const std::string &_type,
   else if (_type == "map" || _type == "image")
     shape.reset(new MapShape(collision));
   else if (_type == "ray")
+  {
     if (_collision)
       shape.reset(new ODERayShape(collision));
     else
-      shape.reset(new ODERayShape(this->world->GetPhysicsEngine()));
+      shape.reset(new ODERayShape(this->world->Physics()));
+  }
   else
     gzerr << "Unable to create collision of type[" << _type << "]\n";
 
@@ -845,23 +862,23 @@ void ODEPhysics::ConvertMass(void *_engineMass, InertialPtr _inertial)
 {
   dMass *odeMass = static_cast<dMass*>(_engineMass);
 
-  odeMass->mass = _inertial->GetMass();
-  odeMass->c[0] = _inertial->GetCoG()[0];
-  odeMass->c[1] = _inertial->GetCoG()[1];
-  odeMass->c[2] = _inertial->GetCoG()[2];
+  odeMass->mass = _inertial->Mass();
+  odeMass->c[0] = _inertial->CoG()[0];
+  odeMass->c[1] = _inertial->CoG()[1];
+  odeMass->c[2] = _inertial->CoG()[2];
 
-  odeMass->I[0*4+0] = _inertial->GetPrincipalMoments()[0];
-  odeMass->I[1*4+1] = _inertial->GetPrincipalMoments()[1];
-  odeMass->I[2*4+2] = _inertial->GetPrincipalMoments()[2];
+  odeMass->I[0*4+0] = _inertial->PrincipalMoments()[0];
+  odeMass->I[1*4+1] = _inertial->PrincipalMoments()[1];
+  odeMass->I[2*4+2] = _inertial->PrincipalMoments()[2];
 
-  odeMass->I[0*4+1] = _inertial->GetProductsofInertia()[0];
-  odeMass->I[1*4+0] = _inertial->GetProductsofInertia()[0];
+  odeMass->I[0*4+1] = _inertial->ProductsOfInertia()[0];
+  odeMass->I[1*4+0] = _inertial->ProductsOfInertia()[0];
 
-  odeMass->I[0*4+2] = _inertial->GetProductsofInertia()[1];
-  odeMass->I[1*4+0] = _inertial->GetProductsofInertia()[1];
+  odeMass->I[0*4+2] = _inertial->ProductsOfInertia()[1];
+  odeMass->I[1*4+0] = _inertial->ProductsOfInertia()[1];
 
-  odeMass->I[1*4+2] = _inertial->GetProductsofInertia()[2];
-  odeMass->I[2*4+1] = _inertial->GetProductsofInertia()[2];
+  odeMass->I[1*4+2] = _inertial->ProductsOfInertia()[2];
+  odeMass->I[2*4+1] = _inertial->ProductsOfInertia()[2];
 }
 
 //////////////////////////////////////////////////
@@ -922,15 +939,17 @@ void ODEPhysics::SetStepType(const std::string &_type)
 }
 
 //////////////////////////////////////////////////
-void ODEPhysics::SetGravity(const gazebo::math::Vector3 &_gravity)
+void ODEPhysics::SetGravity(const ignition::math::Vector3d &_gravity)
 {
-  this->world->SetGravitySDF(_gravity.Ign());
-  dWorldSetGravity(this->dataPtr->worldId, _gravity.x, _gravity.y, _gravity.z);
+  this->world->SetGravitySDF(_gravity);
+  dWorldSetGravity(this->dataPtr->worldId, _gravity.X(), _gravity.Y(),
+      _gravity.Z());
 }
 
 //////////////////////////////////////////////////
 void ODEPhysics::CollisionCallback(void *_data, dGeomID _o1, dGeomID _o2)
 {
+  IGN_PROFILE("ODEPhysics::CollisionCallback");
   dBodyID b1 = dGeomGetBody(_o1);
   dBodyID b2 = dGeomGetBody(_o2);
 
@@ -948,18 +967,8 @@ void ODEPhysics::CollisionCallback(void *_data, dGeomID _o1, dGeomID _o2)
   }
   else
   {
-    ODECollision *collision1 = NULL;
-    ODECollision *collision2 = NULL;
-
-    // Exit if both bodies are not enabled
-    if (dGeomGetCategoryBits(_o1) != GZ_SENSOR_COLLIDE &&
-        dGeomGetCategoryBits(_o2) != GZ_SENSOR_COLLIDE &&
-        ((b1 && b2 && !dBodyIsEnabled(b1) && !dBodyIsEnabled(b2)) ||
-        (!b2 && b1 && !dBodyIsEnabled(b1)) ||
-        (!b1 && b2 && !dBodyIsEnabled(b2))))
-    {
-      return;
-    }
+    ODECollision *collision1 = nullptr;
+    ODECollision *collision2 = nullptr;
 
     // Get pointers to the underlying collisions
     if (dGeomGetClass(_o1) == dGeomTransformClass)
@@ -974,6 +983,18 @@ void ODEPhysics::CollisionCallback(void *_data, dGeomID _o1, dGeomID _o2)
     else
       collision2 = static_cast<ODECollision*>(dGeomGetData(_o2));
 
+    // Exit if both bodies are not enabled
+    if (dGeomGetCategoryBits(_o1) != GZ_SENSOR_COLLIDE &&
+        dGeomGetCategoryBits(_o2) != GZ_SENSOR_COLLIDE &&
+        !self->contactManager->NeverDropContacts() &&
+        !self->contactManager->SubscribersConnected(collision1, collision2) &&
+        ((b1 && b2 && !dBodyIsEnabled(b1) && !dBodyIsEnabled(b2)) ||
+        (!b2 && b1 && !dBodyIsEnabled(b1)) ||
+        (!b1 && b2 && !dBodyIsEnabled(b2))))
+    {
+      return;
+    }
+
     // Make sure both collision pointers are valid.
     if (collision1 && collision2)
     {
@@ -1056,7 +1077,7 @@ void ODEPhysics::Collide(ODECollision *_collision1, ODECollision *_collision2,
     this->dataPtr->indices[i] = i;
 
   // Choose only the best contacts if too many were generated.
-  if (numc > maxCollide)
+  if (maxCollide > 0 && numc > maxCollide)
   {
     double max = _contactCollisions[maxCollide-1].depth;
     for (unsigned int i = maxCollide; i < numc; ++i)
@@ -1101,11 +1122,11 @@ void ODEPhysics::Collide(ODECollision *_collision1, ODECollision *_collision2,
   //                                _collision2->surface->softCFM);
 
   // assign fdir1 if not set as 0
-  math::Vector3 fd = surf1->FrictionPyramid()->direction1;
-  if (fd != math::Vector3::Zero)
+  ignition::math::Vector3d fd = surf1->FrictionPyramid()->direction1;
+  if (fd != ignition::math::Vector3d::Zero)
   {
     // fdir1 is in body local frame, rotate it into world frame
-    fd = _collision1->GetWorldPose().rot.RotateVector(fd);
+    fd = _collision1->WorldPose().Rot().RotateVector(fd);
   }
 
   /// \TODO: Better treatment when both surfaces have fdir1 specified.
@@ -1115,29 +1136,31 @@ void ODEPhysics::Collide(ODECollision *_collision1, ODECollision *_collision2,
   /// As a hack, we'll simply compare mu1 from
   /// both surfaces for now, and use fdir1 specified by
   /// surface with smaller mu1.
-  math::Vector3 fd2 = surf2->FrictionPyramid()->direction1;
-  if (fd2 != math::Vector3::Zero && (fd == math::Vector3::Zero ||
+  ignition::math::Vector3d fd2 = surf2->FrictionPyramid()->direction1;
+  if (fd2 != ignition::math::Vector3d::Zero &&
+      (fd == ignition::math::Vector3d::Zero ||
         surf1->FrictionPyramid()->MuPrimary() >
         surf2->FrictionPyramid()->MuPrimary()))
   {
     // fdir1 is in body local frame, rotate it into world frame
-    fd2 = _collision2->GetWorldPose().rot.RotateVector(fd2);
+    fd2 = _collision2->WorldPose().Rot().RotateVector(fd2);
 
     /// \TODO: uncomment gzlog below once we confirm it does not affect
     /// performance
-    /// if (fd2 != math::Vector3::Zero && fd != math::Vector3::Zero &&
+    /// if (fd2 != ignition::math::Vector3d::Zero &&
+    ///       fd != ignition::math::Vector3d::Zero &&
     ///       _collision1->surface->mu1 > _collision2->surface->mu1)
     ///   gzlog << "both contact surfaces have non-zero fdir1, comparing"
     ///         << " comparing mu1 from both surfaces, and use fdir1"
     ///         << " from surface with smaller mu1\n";
   }
 
-  if (fd != math::Vector3::Zero)
+  if (fd != ignition::math::Vector3d::Zero)
   {
     contact.surface.mode |= dContactFDir1;
-    contact.fdir1[0] = fd.x;
-    contact.fdir1[1] = fd.y;
-    contact.fdir1[2] = fd.z;
+    contact.fdir1[0] = fd.X();
+    contact.fdir1[1] = fd.Y();
+    contact.fdir1[2] = fd.Z();
   }
 
   // Set the friction coefficients.
@@ -1148,13 +1171,21 @@ void ODEPhysics::Collide(ODECollision *_collision1, ODECollision *_collision2,
   contact.surface.mu3 = std::min(surf1->FrictionPyramid()->MuTorsion(),
                                  surf2->FrictionPyramid()->MuTorsion());
 
-  // Set the slip values
-  contact.surface.slip1 = std::min(surf1->slip1,
-                                   surf2->slip1);
-  contact.surface.slip2 = std::min(surf1->slip2,
-                                   surf2->slip2);
-  contact.surface.slip3 = std::min(surf1->slipTorsion,
-                                   surf2->slipTorsion);
+  // Combine the slip values
+  // The slip is equivalent to the inverse of a viscous damping term
+  // To combine dampers in series, the inverse of damping is summed
+  // So the sum of slip parameters is used to combine them
+  contact.surface.slip1 = surf1->slip1 + surf2->slip1;
+  contact.surface.slip2 = surf1->slip2 + surf2->slip2;
+  contact.surface.slip3 = surf1->slipTorsion + surf2->slipTorsion;
+  // The slip parameter acts like a damper at each contact point
+  // so the total damping for each collision is multiplied by the
+  // number of contact points (numc).
+  // To eliminate this dependence on numc, the inverse damping
+  // is multipled by numc.
+  contact.surface.slip1 *= numc;
+  contact.surface.slip2 *= numc;
+  contact.surface.slip3 *= numc;
 
   // Combine torsional friction patch radius values
   contact.surface.patch_radius =
@@ -1233,12 +1264,12 @@ void ODEPhysics::Collide(ODECollision *_collision1, ODECollision *_collision2,
   dBodyID b1 = dGeomGetBody(_collision1->GetCollisionId());
   dBodyID b2 = dGeomGetBody(_collision2->GetCollisionId());
 
-  // Add a new contact to the manager. This will return NULL if no one is
+  // Add a new contact to the manager. This will return nullptr if no one is
   // listening for contact information.
   Contact *contactFeedback = this->contactManager->NewContact(_collision1,
-      _collision2, this->world->GetSimTime());
+      _collision2, this->world->SimTime());
 
-  ODEJointFeedback *jointFeedback = NULL;
+  ODEJointFeedback *jointFeedback = nullptr;
 
   // Create a joint feedback mechanism
   if (contactFeedback)
@@ -1340,18 +1371,18 @@ void ODEPhysics::DebugPrint() const
   {
     b = dWorldGetBody(this->dataPtr->worldId, i);
     ODELink *link = static_cast<ODELink*>(dBodyGetData(b));
-    math::Pose pose = link->GetWorldPose();
+    ignition::math::Pose3d pose = link->WorldPose();
     const dReal *pos = dBodyGetPosition(b);
     const dReal *rot = dBodyGetRotation(b);
-    math::Vector3 dpos(pos[0], pos[1], pos[2]);
-    math::Quaternion drot(rot[0], rot[1], rot[2], rot[3]);
+    ignition::math::Vector3d dpos(pos[0], pos[1], pos[2]);
+    ignition::math::Quaterniond drot(rot[0], rot[1], rot[2], rot[3]);
 
     std::cout << "Body[" << link->GetScopedName() << "]\n";
     std::cout << "  World: Pos[" << dpos << "] Rot[" << drot << "]\n";
-    if (pose.pos != dpos)
-      std::cout << "    Incorrect world pos[" << pose.pos << "]\n";
-    if (pose.rot != drot)
-      std::cout << "    Incorrect world rot[" << pose.rot << "]\n";
+    if (pose.Pos() != dpos)
+      std::cout << "    Incorrect world pos[" << pose.Pos() << "]\n";
+    if (pose.Rot() != drot)
+      std::cout << "    Incorrect world rot[" << pose.Rot() << "]\n";
 
     dMass mass;
     dBodyGetMass(b, &mass);
@@ -1363,7 +1394,7 @@ void ODEPhysics::DebugPrint() const
     {
       ODECollision *coll = static_cast<ODECollision*>(dGeomGetData(g));
 
-      pose = coll->GetWorldPose();
+      pose = coll->WorldPose();
       const dReal *gpos = dGeomGetPosition(g);
       const dReal *grot = dGeomGetRotation(g);
       dpos.Set(gpos[0], gpos[1], gpos[2]);
@@ -1372,10 +1403,10 @@ void ODEPhysics::DebugPrint() const
       std::cout << "    Geom[" << coll->GetScopedName() << "]\n";
       std::cout << "      World: Pos[" << dpos << "] Rot[" << drot << "]\n";
 
-      if (pose.pos != dpos)
-        std::cout << "      Incorrect world pos[" << pose.pos << "]\n";
-      if (pose.rot != drot)
-        std::cout << "      Incorrect world rot[" << pose.rot << "]\n";
+      if (pose.Pos() != dpos)
+        std::cout << "      Incorrect world pos[" << pose.Pos() << "]\n";
+      if (pose.Rot() != drot)
+        std::cout << "      Incorrect world rot[" << pose.Rot() << "]\n";
 
       g = dBodyGetNextGeom(g);
     }
@@ -1392,88 +1423,88 @@ void ODEPhysics::SetSeed(uint32_t _seed)
 bool ODEPhysics::SetParam(const std::string &_key, const boost::any &_value)
 {
   sdf::ElementPtr odeElem = this->sdf->GetElement("ode");
-  GZ_ASSERT(odeElem != NULL, "ODE SDF element does not exist");
+  GZ_ASSERT(odeElem != nullptr, "ODE SDF element does not exist");
 
   try
   {
     if (_key == "solver_type")
     {
-      this->SetStepType(boost::any_cast<std::string>(_value));
+      this->SetStepType(any_cast<std::string>(_value));
     }
     else if (_key == "cfm")
     {
-      double value = boost::any_cast<double>(_value);
+      double value = any_cast<double>(_value);
       odeElem->GetElement("constraints")->GetElement("cfm")->Set(value);
       dWorldSetCFM(this->dataPtr->worldId, value);
     }
     else if (_key == "erp")
     {
-      double value = boost::any_cast<double>(_value);
+      double value = any_cast<double>(_value);
       odeElem->GetElement("constraints")->GetElement("erp")->Set(value);
       dWorldSetERP(this->dataPtr->worldId, value);
     }
     else if (_key == "precon_iters")
     {
-      int value = boost::any_cast<int>(_value);
+      int value = any_cast<int>(_value);
       odeElem->GetElement("solver")->GetElement("precon_iters")->Set(value);
       dWorldSetQuickStepPreconIterations(this->dataPtr->worldId, value);
     }
     else if (_key == "iters")
     {
-      int value = boost::any_cast<int>(_value);
+      int value = any_cast<int>(_value);
       odeElem->GetElement("solver")->GetElement("iters")->Set(value);
       dWorldSetQuickStepNumIterations(this->dataPtr->worldId, value);
     }
     else if (_key == "sor")
     {
-      double value = boost::any_cast<double>(_value);
+      double value = any_cast<double>(_value);
       odeElem->GetElement("solver")->GetElement("sor")->Set(value);
       dWorldSetQuickStepW(this->dataPtr->worldId, value);
     }
     else if (_key == "friction_model")
-      this->SetFrictionModel(boost::any_cast<std::string>(_value));
+      this->SetFrictionModel(any_cast<std::string>(_value));
     else if (_key == "world_step_solver")
-      this->SetWorldStepSolverType(boost::any_cast<std::string>(_value));
+      this->SetWorldStepSolverType(any_cast<std::string>(_value));
     else if (_key == "contact_max_correcting_vel")
     {
-      double value = boost::any_cast<double>(_value);
+      double value = any_cast<double>(_value);
       odeElem->GetElement("constraints")->GetElement(
           "contact_max_correcting_vel")->Set(value);
       dWorldSetContactMaxCorrectingVel(this->dataPtr->worldId, value);
     }
     else if (_key == "contact_surface_layer")
     {
-      double value = boost::any_cast<double>(_value);
+      double value = any_cast<double>(_value);
       odeElem->GetElement("constraints")->GetElement(
           "contact_surface_layer")->Set(value);
       dWorldSetContactSurfaceLayer(this->dataPtr->worldId, value);
     }
     else if (_key == "max_contacts")
     {
-      int value = boost::any_cast<int>(_value);
+      int value = any_cast<int>(_value);
       this->sdf->GetElement("max_contacts")->GetValue()->Set(value);
     }
     else if (_key == "min_step_size")
     {
       /// TODO: Implement min step size param
-      double value = boost::any_cast<double>(_value);
+      double value = any_cast<double>(_value);
       odeElem->GetElement("solver")->GetElement("min_step_size")->Set(value);
     }
     else if (_key == "sor_lcp_tolerance")
     {
       dWorldSetQuickStepTolerance(this->dataPtr->worldId,
-          boost::any_cast<double>(_value));
+          any_cast<double>(_value));
     }
     else if (_key == "rms_error_tolerance")
     {
       gzwarn << "please use sor_lcp_tolerance in the future.\n";
       dWorldSetQuickStepTolerance(this->dataPtr->worldId,
-          boost::any_cast<double>(_value));
+          any_cast<double>(_value));
     }
     else if (_key == "inertia_ratio_reduction" ||
              _key == "use_dynamic_moi_rescaling")
     {
-      bool value = boost::any_cast<bool>(_value);
+      bool value = any_cast<bool>(_value);
       dWorldSetQuickStepInertiaRatioReduction(this->dataPtr->worldId, value);
       if (odeElem->GetElement("solver")->HasElement(
             "use_dynamic_moi_rescaling"))
@@ -1485,36 +1516,68 @@ bool ODEPhysics::SetParam(const std::string &_key, const boost::any &_value)
     else if (_key == "contact_residual_smoothing")
     {
       dWorldSetQuickStepContactResidualSmoothing(this->dataPtr->worldId,
-        boost::any_cast<double>(_value));
+        any_cast<double>(_value));
     }
     else if (_key == "contact_sor_scale")
     {
       dWorldSetQuickStepContactSORScalingFactor(this->dataPtr->worldId,
-        boost::any_cast<double>(_value));
+        any_cast<double>(_value));
     }
     else if (_key == "thread_position_correction")
     {
       dWorldSetQuickStepThreadPositionCorrection(this->dataPtr->worldId,
-        boost::any_cast<bool>(_value));
+        any_cast<bool>(_value));
     }
     else if (_key == "experimental_row_reordering")
     {
       dWorldSetQuickStepExperimentalRowReordering(this->dataPtr->worldId,
-        boost::any_cast<bool>(_value));
+        any_cast<bool>(_value));
     }
     else if (_key == "warm_start_factor")
     {
       dWorldSetQuickStepWarmStartFactor(this->dataPtr->worldId,
-        boost::any_cast<double>(_value));
+        any_cast<double>(_value));
     }
     else if (_key == "extra_friction_iterations")
     {
       dWorldSetQuickStepExtraFrictionIterations(this->dataPtr->worldId,
-        boost::any_cast<int>(_value));
+        any_cast<int>(_value));
+    }
+    else if (_key == "island_threads")
+    {
+      int value;
+      try
+      {
+        value = any_cast<int>(_value);
+      }
+      catch(const boost::bad_any_cast &e)
+      {
+        gzerr << "boost any_cast error:" << e.what() << "\n";
+        return false;
+      }
+      dWorldSetIslandThreads(this->dataPtr->worldId, value);
     }
     else if (_key == "ode_quiet")
     {
-      bool odeQuiet = boost::any_cast<bool>(_value);
+      bool odeQuiet;
+      try
+      {
+        odeQuiet = any_cast<bool>(_value);
+      }
+      catch(std::bad_any_cast &)
+      {
+        // If added to an SDFormat world file, this parameter will be
+        // encoded as a string, since it is not part of the SDFormat spec.
+        // In order to parse it, define an sdf::Param with type string
+        // and call Get<bool> to use libsdformat's existing logic for this.
+        // copied from sdformat's Param_TEST.cc
+        sdf::Param strParam("key", "string", "false", false, "description");
+        // cast to std::string and set the sdf::Param with this value
+        strParam.Set(any_cast<std::string>(_value));
+        // get the string value as bool
+        strParam.Get<bool>(odeQuiet);
+      }
+
       if (odeQuiet)
       {
         dSetMessageHandler(&dMessageQuiet);
@@ -1529,9 +1592,15 @@ bool ODEPhysics::SetParam(const std::string &_key, const boost::any &_value)
       return PhysicsEngine::SetParam(_key, _value);
     }
   }
+  catch(std::bad_any_cast &e)
+  {
+    gzerr << "SetParam(" << _key << ") std::any_cast error: "
+          << e.what() << std::endl;
+    return false;
+  }
   catch(boost::bad_any_cast &e)
   {
-    gzerr << "ODEPhysics::SetParam(" << _key << ") boost::any_cast error: "
+    gzerr << "SetParam(" << _key << ") boost::any_cast error: "
           << e.what() << std::endl;
     return false;
   }
@@ -1550,7 +1619,7 @@ boost::any ODEPhysics::GetParam(const std::string &_key) const
 bool ODEPhysics::GetParam(const std::string &_key, boost::any &_value) const
 {
   sdf::ElementPtr odeElem = this->sdf->GetElement("ode");
-  GZ_ASSERT(odeElem != NULL, "ODE SDF element does not exist");
+  GZ_ASSERT(odeElem != nullptr, "ODE SDF element does not exist");
 
   if (_key == "solver_type")
   {
@@ -1611,6 +1680,8 @@ bool ODEPhysics::GetParam(const std::string &_key, boost::any &_value) const
     _value = dWorldGetQuickStepExtraFrictionIterations(this->dataPtr->worldId);
   else if (_key == "friction_model")
     _value = this->GetFrictionModel();
+  else if (_key == "island_threads")
+    _value = dWorldGetIslandThreads(this->dataPtr->worldId);
   else if (_key == "ode_quiet")
     _value = dGetMessageHandler() != 0;
   else if (_key == "world_step_solver")
